@@ -15,20 +15,20 @@ fi
 srcdir=$wadir
 ver=`cat "$wadir/version"`
 
-# Find temp directory
-if [ "$tempdir" = "" ]; then
-	tempdir=$tempdir
-fi
-
 if [ $? != "0" ]; then
 	echo "ERROR: Cannot find the Usermin install directory";
 	echo "";
 	exit 1;
 fi
 
-echo "***********************************************************************"
-echo "*            Welcome to the Usermin setup script, version $ver       *"
-echo "***********************************************************************"
+# Find temp directory
+if [ "$tempdir" = "" ]; then
+	tempdir=/tmp/.webmin
+fi
+
+echo "****************************************************************************"
+echo "          Welcome to the Usermin setup script, version $ver"
+echo "****************************************************************************"
 echo "Usermin is a web-based interface that allows Unix-like operating"
 echo "systems and common Unix services to be easily administered."
 echo ""
@@ -47,7 +47,7 @@ fi
 # Use the supplied destination directory, if any
 if [ "$1" != "" ]; then
 	wadir=$1
-	echo "Installing Usermin from $srcdir to $wadir ..."
+	echo "Installing Usermin from $srcdir to $wadir"
 	if [ ! -d "$wadir" ]; then
 		mkdir "$wadir"
 		if [ "$?" != "0" ]; then
@@ -65,10 +65,14 @@ if [ "$1" != "" ]; then
 		fi
 	fi
 else
-	echo "Installing Usermin in $wadir ..."
+	echo "Installing Usermin in $wadir"
 fi
 cd "$wadir"
 
+# Export Usermin root dir
+WEBMIN_LIBDIR=$wadir
+export WEBMIN_LIBDIR
+
 # Display install directory
 allmods=`cd "$srcdir"; echo */module.info | sed -e 's/\/module.info//g'`
 if [ "$allmods" = "" ]; then
@@ -84,7 +88,7 @@ if [ -r "$srcdir/setup-pre.sh" ]; then
 fi
 
 # Ask for usermin config directory
-echo "***********************************************************************"
+echo "****************************************************************************"
 echo "Usermin uses separate directories for configuration files and log files."
 echo "Unless you want to run multiple versions of Usermin at the same time"
 echo "you can just accept the defaults."
@@ -111,13 +115,14 @@ if [ ! -d $config_dir ]; then
 	fi
 fi
 if [ -r "$config_dir/config" ]; then
-	echo "Found existing Usermin configuration in $config_dir"
-	echo ""
+	echo ".. found"
 	upgrading=1
+else
+	userminrunning=1
 fi
 
 # Check if upgrading from an old version
-if [ "$upgrading" = 1 ]; then
+if [ "$upgrading" = "1" ]; then
 	echo ""
 
 	# Get current var path
@@ -149,8 +154,12 @@ if [ "$upgrading" = 1 ]; then
 	ssl=`grep "^ssl=" $config_dir/miniserv.conf | sed -e 's/ssl=//g'`
 	atboot=`grep "^atboot=" $config_dir/miniserv.conf | sed -e 's/atboot=//g'`
 	inetd=`grep "^inetd=" $config_dir/miniserv.conf | sed -e 's/inetd=//g'`
+	userminpidfile=`grep "^pidfile=" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'`
+	if [ -r "$userminpidfile" ]; then
+		userminrunning=1
+	fi
 
-	if [ "$inetd" != "1" ]; then
+	if [ "$upgrading" = "1" -a "$inetd" != "1" -a "$nostop" = "" ]; then
 		# Stop old version
 		$config_dir/stop >/dev/null 2>&1
 	fi
@@ -159,7 +168,7 @@ if [ "$upgrading" = 1 ]; then
 	if [ "$wadir" != "$srcdir" ]; then
 		echo "Copying files to $wadir .."
 		(cd "$srcdir" ; tar cf - . | (cd "$wadir" ; tar xf -))
-		echo "..done"
+		echo ".. done"
 		echo ""
 	fi
 
@@ -184,7 +193,7 @@ if [ "$upgrading" = 1 ]; then
 			autothird=1
 		fi
 		$perl "$wadir/thirdparty.pl" "$wadir" "$oldwadir" $autothird
-		echo "..done"
+		echo ".. done"
 		echo ""
 	fi
 
@@ -215,7 +224,7 @@ else
 	fi
 	if [ "$var_dir" = "/" ]; then
 		echo "Log directory cannot be /"
-		exit ""
+		echo ""
 		exit 3
 	fi
 	if [ ! -d $var_dir ]; then
@@ -229,7 +238,7 @@ else
 	echo ""
 
 	# Ask where perl is installed
-	echo "***********************************************************************"
+	echo "****************************************************************************"
 	echo "Usermin is written entirely in Perl. Please enter the full path to the"
 	echo "Perl 5 interpreter on your system."
 	echo ""
@@ -260,7 +269,7 @@ else
 	echo ""
 
 	# Test perl 
-	echo "Testing Perl ..."
+	echo "Testing Perl .."
 	if [ ! -x $perl ]; then
 		echo "ERROR: Failed to find perl at $perl"
 		echo ""
@@ -297,7 +306,7 @@ else
 		echo ""
 		exit 8
 	fi
-	echo "Perl seems to be installed ok"
+	echo ".. done"
 	echo ""
 
 	# Create temp files directory
@@ -309,7 +318,7 @@ else
 	fi
 
 	# Ask for operating system type
-	echo "***********************************************************************"
+	echo "****************************************************************************"
 	if [ "$os_type" = "" ]; then
 		if [ "$autoos" = "" ]; then
 			autoos=2
@@ -326,12 +335,12 @@ else
 	echo ""
 
 	# Ask for web server port, name and password
-	echo "***********************************************************************"
+	echo "****************************************************************************"
 	echo "Usermin uses its own password protected web server to provide access"
 	echo "to the administration programs. The setup script needs to know :"
 	echo " - What port to run the web server on. There must not be another"
 	echo "   web server already using this port."
-	echo " - If the webserver should use SSL (if your system supports it)."
+	echo " - If the web server should use SSL (if your system supports it)."
 	echo ""
 	printf "Web server port (default 20000): "
 	if [ "$port" = "" ]; then
@@ -359,7 +368,7 @@ else
 			exit 13
 		fi
 	fi
-
+	echo ""
 	# Ask the user if SSL should be used
 	if [ "$ssl" = "" ]; then
 		ssl=0
@@ -385,18 +394,18 @@ else
 	fi
 
 	# Copy files to target directory
-	echo "***********************************************************************"
+	echo "****************************************************************************"
 	if [ "$wadir" != "$srcdir" ]; then
 		echo "Copying files to $wadir .."
 		(cd "$srcdir" ; tar cf - . | (cd "$wadir" ; tar xf -))
-		echo "..done"
+		echo ".. done"
 		echo ""
 	fi
 
 	# Create webserver config file
 	echo $perl > $config_dir/perl-path
 	echo $var_dir > $config_dir/var-path
-	echo "Creating web server config files.."
+	echo "Creating web server config files .."
 	cfile=$config_dir/miniserv.conf
 	echo "port=$port" >> $cfile
 	echo "root=$wadir" >> $cfile
@@ -412,7 +421,7 @@ else
 	echo "no_ssl2=1" >> $cfile
 	echo "no_ssl3=1" >> $cfile
 	openssl version 2>&1 | grep "OpenSSL 1" 2>&1 >/dev/null
-	if [ "$?" == "0" ]; then
+	if [ "$?" = "0" ]; then
 		echo "no_tls1=1" >> $cfile
 		echo "no_tls1_1=1" >> $cfile    
 	fi
@@ -452,12 +461,22 @@ else
 	openssl version >/dev/null 2>&1
 	if [ "$?" = "0" ]; then
 		# We can generate a new SSL key for this host
-		host=`hostname`
-		openssl req -newkey rsa:2048 -x509 -nodes -out "$tempdir/cert" -keyout "$tempdir/key" -days 1825 -sha256 >/dev/null 2>&1 <<EOF
+		host=`hostname 2>/dev/null`
+		if [ "$host" = "" ]; then
+			host=`uname -n 2>/dev/null`
+		fi
+
+		# OpenSSL support `-addext` flag?
+		addtextsup="-addext subjectAltName=DNS:$host,DNS:localhost -addext extendedKeyUsage=serverAuth"
+		openssl version 2>&1 | grep "OpenSSL 1.0" >/dev/null
+		if [ "$?" = "0" ]; then
+			addtextsup=""
+		fi
+		openssl req -newkey rsa:2048 -x509 -nodes -out $tempdir/cert -keyout $tempdir/key -days 1825 -sha256 -subj "/CN=$host/C=US/L=Santa Clara" $addtextsup >/dev/null 2>&1 <<EOF
 .
 .
 .
-Usermin Webserver on $host
+Usermin web server on $host
 .
 *
 root@$host
@@ -475,10 +494,10 @@ EOF
 	echo "keyfile=$config_dir/miniserv.pem" >> $cfile
 
 	chmod 644 $cfile
-	echo "..done"
+	echo ".. done"
 	echo ""
 
-	echo "Creating access control file.."
+	echo "Creating access control file .."
 	defmods=`cat "$wadir/defaultmodules" 2>/dev/null`
 	if [ "$defmods" = "" ]; then
 		defmods="$allmods"
@@ -487,54 +506,149 @@ EOF
 	rm -f $afile
 	echo "user: $defmods" >> $afile
 	chmod 644 $afile
-	echo "..done"
+	echo ".. done"
 	echo ""
 
 fi
 
 if [ "$noperlpath" = "" ]; then
-	echo "Inserting path to perl into scripts.."
+	echo "Inserting path to perl into scripts .."
 	(find "$wadir" -name '*.cgi' -print ; find "$wadir" -name '*.pl' -print) | $perl "$wadir/perlpath.pl" $perl -
-	echo "..done"
+	echo ".. done"
 	echo ""
 fi
 
-echo "Creating start and stop scripts.."
-rm -f $config_dir/stop $config_dir/start
-echo "#!/bin/sh" >>$config_dir/start
-echo "echo Starting Usermin server in $wadir" >>$config_dir/start
-echo "trap '' 1" >>$config_dir/start
-echo "LANG=" >>$config_dir/start
-echo "export LANG" >>$config_dir/start
-echo "#PERLIO=:raw" >>$config_dir/start
-echo "unset PERLIO" >>$config_dir/start
-echo "export PERLIO" >>$config_dir/start
+# Re-generating main scripts
+echo "Creating start and stop init scripts .."
+# Start main
+echo "#!/bin/sh" >$config_dir/.start-init
+echo "echo Starting Usermin server in $wadir" >>$config_dir/.start-init
+echo "trap '' 1" >>$config_dir/.start-init
+echo "LANG=" >>$config_dir/.start-init
+echo "export LANG" >>$config_dir/.start-init
+echo "unset PERLIO" >>$config_dir/.start-init
+echo "export PERLIO" >>$config_dir/.start-init
+echo "PERLLIB=$PERLLIB" >>$config_dir/.start-init
+echo "export PERLLIB" >>$config_dir/.start-init
 uname -a | grep -i 'HP/*UX' >/dev/null
 if [ $? = "0" ]; then
-	echo "exec "$wadir/miniserv.pl" $config_dir/miniserv.conf &" >>$config_dir/start
+	echo "exec '$wadir/miniserv.pl' \$* $config_dir/miniserv.conf &" >>$config_dir/.start-init
 else
-	uname -a | grep -i FreeBSD >/dev/null
-	if [ "$?" = "0" ]; then
-		echo "LD_PRELOAD=`echo /usr/lib/libpam.so.?`" >>$config_dir/start
-		echo "export LD_PRELOAD" >>$config_dir/start
-	fi
-	echo "exec "$wadir/miniserv.pl" $config_dir/miniserv.conf" >>$config_dir/start
+	echo "exec '$wadir/miniserv.pl' \$* $config_dir/miniserv.conf" >>$config_dir/.start-init
 fi
+# Stop main
+echo "#!/bin/sh" >$config_dir/.stop-init
+echo "if [ \"\$1\" = \"--kill\" ]; then" >>$config_dir/.stop-init
+echo "  echo Force stopping Usermin server in $wadir" >>$config_dir/.stop-init
+echo "else" >>$config_dir/.stop-init
+echo "  echo Stopping Usermin server in $wadir" >>$config_dir/.stop-init
+echo "fi" >>$config_dir/.stop-init
+echo "pidfile=\`grep \"^pidfile=\" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'\`" >>$config_dir/.stop-init
+echo "pid=\`cat \$pidfile\` 2>/dev/null" >>$config_dir/.stop-init
+echo "if [ \"\$pid\" != \"\" ]; then" >>$config_dir/.stop-init
+echo "  kill \$pid || exit 1" >>$config_dir/.stop-init
+echo "  touch $var_dir/stop-flag" >>$config_dir/.stop-init
+echo "  if [ \"\$1\" = \"--kill\" ]; then" >>$config_dir/.stop-init
+echo "    sleep 1" >>$config_dir/.stop-init
+echo "    ((ps axf | grep \"usermin\/miniserv\.pl\" | awk '{print \"kill -9 -- -\" \$1}' | bash) || kill -9 -- -\$pid || kill -9 \$pid) 2>/dev/null" >>$config_dir/.stop-init
+echo "  fi" >>$config_dir/.stop-init
+echo "  exit 0" >>$config_dir/.stop-init
+echo "else" >>$config_dir/.stop-init
+echo "  if [ \"\$1\" = \"--kill\" ]; then" >>$config_dir/.stop-init
+echo "    (ps axf | grep \"usermin\/miniserv\.pl\" | awk '{print \"kill -9 -- -\" \$1}' | bash) 2>/dev/null" >>$config_dir/.stop-init
+echo "  fi" >>$config_dir/.stop-init
+echo "fi" >>$config_dir/.stop-init
+# Restart main
+echo "#!/bin/sh" >$config_dir/.restart-init
+echo "$config_dir/.stop-init" >>$config_dir/.restart-init
+echo "$config_dir/.start-init" >>$config_dir/.restart-init
+# Force reload main
+echo "#!/bin/sh" >$config_dir/.restart-by-force-kill-init
+echo "$config_dir/.stop-init --kill" >>$config_dir/.restart-by-force-kill-init
+echo "$config_dir/.start-init" >>$config_dir/.restart-by-force-kill-init
+# Reload main
+echo "#!/bin/sh" >$config_dir/.reload-init
+echo "echo Reloading Usermin server in $wadir" >>$config_dir/.reload-init
+echo "pidfile=\`grep \"^pidfile=\" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'\`" >>$config_dir/.reload-init
+echo "kill -USR1 \`cat \$pidfile\`" >>$config_dir/.reload-init
+# Pre install
+echo "#!/bin/sh" >$config_dir/.pre-install
+echo "$config_dir/.stop-init" >>$config_dir/.pre-install
+# Post install
+echo "#!/bin/sh" >$config_dir/.post-install
+echo "$config_dir/.start-init" >>$config_dir/.post-install
 
-echo "#!/bin/sh" >>$config_dir/stop
-echo "echo Stopping Usermin server in $wadir" >>$config_dir/stop
-echo "pidfile=\`grep \"^pidfile=\" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'\`" >>$config_dir/stop
-echo "kill \`cat \$pidfile\`" >>$config_dir/stop
-chmod 755 $config_dir/start $config_dir/stop
-echo "..done"
+
+chmod 755 $config_dir/.stop-init $config_dir/.start-init $config_dir/.restart-init $config_dir/.restart-by-force-kill-init $config_dir/.reload-init $config_dir/.pre-install $config_dir/.post-install
+echo ".. done"
 echo ""
 
-if [ "$upgrading" = 1 ]; then
-	echo "Updating config files.."
+# Re-generating supplementary
+
+# Clear existing
+rm -f $config_dir/stop $config_dir/start $config_dir/restart $config_dir/restart-by-force-kill $config_dir/reload
+
+# Start init.d
+ln -s $config_dir/.start-init $config_dir/start >/dev/null 2>&1
+# Stop init.d
+ln -s $config_dir/.stop-init $config_dir/stop >/dev/null 2>&1
+# Restart init.d
+ln -s $config_dir/.restart-init $config_dir/restart >/dev/null 2>&1
+# Force reload init.d
+ln -s $config_dir/.restart-by-force-kill-init $config_dir/restart-by-force-kill >/dev/null 2>&1
+# Reload init.d
+ln -s $config_dir/.reload-init $config_dir/reload >/dev/null 2>&1
+
+# For systemd create different start/stop scripts
+systemctlcmd=`which systemctl 2>/dev/null`
+if [ -x "$systemctlcmd" ]; then
+	rm -f $config_dir/stop $config_dir/start $config_dir/restart $config_dir/restart-by-force-kill $config_dir/reload
+
+	echo "Creating start and stop scripts (systemd) .."
+	# Start systemd
+	echo "#!/bin/sh" >$config_dir/start
+	echo "$systemctlcmd start usermin" >>$config_dir/start
+	# Stop systemd
+	echo "#!/bin/sh" >$config_dir/stop
+	echo "$systemctlcmd stop usermin" >>$config_dir/stop
+	# Restart systemd
+	echo "#!/bin/sh" >$config_dir/restart
+	echo "$systemctlcmd restart usermin" >>$config_dir/restart
+	# Force reload systemd
+	echo "#!/bin/sh" >$config_dir/restart-by-force-kill
+	echo "$systemctlcmd stop usermin" >>$config_dir/restart-by-force-kill
+	echo "$config_dir/.stop-init --kill >/dev/null 2>&1" >>$config_dir/restart-by-force-kill
+	echo "$systemctlcmd start usermin" >>$config_dir/restart-by-force-kill
+	# Reload systemd
+	echo "#!/bin/sh" >$config_dir/reload
+	echo "$systemctlcmd reload usermin" >>$config_dir/reload
+	# Pre-install on systemd
+	echo "#!/bin/sh" >$config_dir/.pre-install
+	echo "$systemctlcmd stop usermin" >>$config_dir/.pre-install
+	# Post-install on systemd
+	echo "#!/bin/sh" >$config_dir/.post-install
+	# echo "$systemctlcmd kill --signal=SIGCONT --kill-who=main usermin" >>$config_dir/.post-install
+	echo "$systemctlcmd start usermin" >>$config_dir/.post-install
+
+	
+	chmod 755 $config_dir/stop $config_dir/start $config_dir/restart $config_dir/restart-by-force-kill $config_dir/reload $config_dir/.pre-install $config_dir/.post-install
 else
-	echo "Copying config files.."
+	# Creating symlinks
+	echo "Creating start and stop init symlinks to scripts .."
 fi
-$perl "$wadir/copyconfig.pl" "$os_type" "$os_version" "$wadir" $config_dir "" $allmods
+echo ".. done"
+echo ""
+
+# Fix existing init file to update start and stop commands
+(cd "$wadir" ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir "$wadir/updateboot.pl")
+cd "$wadir"
+
+if [ "$upgrading" = 1 ]; then
+	echo "Updating config files .."
+else
+	echo "Copying config files .."
+fi
+newmods=`$perl "$wadir/copyconfig.pl" "$os_type" "$os_version" "$wadir" $config_dir "" $allmods`
 if [ "$upgrading" != 1 ]; then
 	# Store the OS and version
 	echo "os_type=$os_type" >> $config_dir/config
@@ -559,7 +673,7 @@ else
 	fi
 fi
 echo $ver > $config_dir/version
-echo "..done"
+echo ".. done"
 echo ""
 
 # Set passwd_ fields in miniserv.conf from global config
@@ -583,6 +697,10 @@ fi
 # Set a special theme if none was set before
 if [ "$theme" = "" ]; then
 	theme=`cat "$wadir/defaulttheme" 2>/dev/null`
+	# If no default theme found fall back to Framed Theme
+	if [ ! -d "$wadir/$theme" ]; then
+		theme="gray-theme"
+	fi
 fi
 oldthemeline=`grep "^theme=" $config_dir/config`
 oldtheme=`echo $oldthemeline | sed -e 's/theme=//g'`
@@ -615,12 +733,21 @@ if [ "\$answer" = "y" ]; then
 	echo "Deleting $wadir .."
 	rm -rf "$wadir"
 	echo "Deleting $config_dir .."
-	rm -rf $config_dir
+	rm -rf "$config_dir"
+	echo "Deleting $var_dir .."
+	rm -rf "$var_dir"
+	systemctlcmd=\`which systemctl 2>/dev/null\`
+	if [ -x "\$systemctlcmd" ]; then
+		echo "Deleting usermin.service .."
+		\$systemctlcmd stop usermin >/dev/null 2>&1 </dev/null
+		rm -f /lib/systemd/system/usermin.service /usr/lib/systemd/system/usermin.service
+		\$systemctlcmd daemon-reload
+	fi
 	echo "Done!"
 fi
 EOF
 	chmod +x $config_dir/uninstall.sh
-	echo "..done"
+	echo ".. done"
 	echo ""
 fi
 
@@ -637,7 +764,7 @@ if [ "$nochown" = "" ]; then
 	fi
 fi
 chmod 600 $config_dir/miniserv.pem 2>/dev/null
-echo "..done"
+echo ".. done"
 echo ""
 
 # Save target directory if one was specified
@@ -647,6 +774,11 @@ else
 	rm -f $config_dir/install-dir
 fi
 
+# Create manifest link for Authentic
+if [ -r "$wadir/authentic-theme/postinstall-usermin.pl" ]; then
+	(cd "$wadir" ; WEBMIN_CONFIG="$config_dir" WEBMIN_VAR="$var_dir" PERLLIB="$wadir" LANG= "$wadir/authentic-theme/postinstall-usermin.pl") >/dev/null 2>&1 </dev/null
+fi
+
 # Run package-defined post-install script
 if [ -r "$srcdir/setup-post.sh" ]; then
 	. "$srcdir/setup-post.sh"
@@ -654,30 +786,58 @@ fi
 
 if [ "$nostart" = "" ]; then
 	if [ "$inetd" != "1" ]; then
-		echo "Attempting to start Usermin mini web server.."
-		$config_dir/start
-		if [ $? != "0" ]; then
-			echo "ERROR: Failed to start web server!"
-			echo ""
-			exit 14
+		action="start"
+		if [ "$upgrading" = "1" ]; then
+			action="restart"
+		fi
+		if [ "$userminrunning" = "1" ]; then
+			echo "Attempting to $action Usermin web server .."
+			$config_dir/$action >/dev/null 2>&1 </dev/null
+			if [ $? != "0" ]; then
+				echo "ERROR: Failed to $action web server!"
+				echo ""
+				exit 14
+			fi
+			echo ".. done"
+		else
+			echo "Skipping to $action Usermin web server (wasn't running) .."
+			echo ".. skipped"
 		fi
-		echo "..done"
 		echo ""
 	fi
-
-	echo "***********************************************************************"
-	echo "Usermin has been installed and started successfully. Use your web"
-	echo "browser to go to"
+	postactionmsg="installed"
+	postactionmsg2="started"
+	if [ "$upgrading" = "1" ]; then
+		postactionmsg="upgraded"
+		postactionmsg2="restarted"
+	fi
+	echo "****************************************************************************"
+	echo "Usermin has been $postactionmsg and $postactionmsg2 successfully."
+	echo ""
+	if [ "$nodepsmsg" = "" -a "$upgrading" != 1 ]; then
+		echo "Since Usermin was installed outside the package manager, ensure the"
+		echo "following recommended Perl modules and packages are present:"
+		echo " Perl modules:"
+		echo "  - DateTime, DateTime::Locale, DateTime::TimeZone, Data::Dumper"
+		echo "  - Digest::MD5, Digest::SHA, Encode::Detect, File::Basename"
+		echo "  - File::Path, Net::SSLeay, Time::HiRes, Time::Local, Time::Piece"
+		echo "  - lib, open"
+		echo " Packages:"
+		echo "  - openssl - Cryptography library with TLS implementation"
+		echo "  - shared-mime-info - Shared MIME information database"
+		echo "  - tar gzip unzip - File compression and packaging utilities"
+		echo ""
+	fi
+	echo "Use your web browser to go to the following URL and login"
+	echo "with the name and password you entered previously:"
 	echo ""
 	host=`hostname`
 	if [ "$ssl" = "1" ]; then
-		echo "  https://$host:$port/"
+		echo "  https://$host:$port"
 	else
-		echo "  http://$host:$port/"
+		echo "  http://$host:$port"
 	fi
 	echo ""
-	echo "and login as any Unix user on your system."
-	echo ""
 	if [ "$ssl" = "1" ]; then
 		echo "Because Usermin uses SSL for encryption only, the certificate"
 		echo "it uses is not signed by one of the recognized CAs such as"
